#---------------------------------------------------------------------
#    Name: WPM_PDtool
# Purpose: Gather system and product information to assist in Problem
#          Determination by the IBM WebSphere Payment Manager Level 2
#          Support group.
#  Author: Robert A. Gibson [rag]
# Version: 1.7
#   Usage: see Usage() below
#
# ToBeDone:
#  - Add disclaimer to code & readme.
#  - Find and format the *CertUtil(Dump|Trace)* files?
#    We may need to look for CertUtil.properies, and parse it, or ask
#    the user for the directory containing these files...
#  - Locate webspherepaymentmanager.log file(s)
#  - Locate WAS *.properties files
#  - Locate httpd.conf file(s)
#  - Locate Dr. Watson log file(s) (generally in C:\Winnt\System32)
# >> Add AIX & Solaris code
#
# History:
#   date  |ver|who|description
# --------+---+---+--------------------------------------------------
# 04/01/01|1.0|rag|New: Windows only version
# 04/15/01|1.1|rag|Add: Check for existance of $TEMP/$pgm file/dir.
#         |   |   |     Replaced Windows Format[Servlet]Trace.cmd
#         |   |   |     code with code to build & execute WPM_Format.cmd
#         |   |   |Add: Code to remove double backslashes.
# 04/26/01|1.2|rag|Fix: PMrootdir(), so it won't infinite loop if
#         |   |   |     WPM isn't found in the registry
#         |   |   |Fix: require "WinRegDump.pl";  So main script can
#         |   |   |     be invoked from any directory on the system.
#         |   |   |Fix: parse of FormatTrace JVM command.  Handle
#         |   |   |     multiple spaces between pieces correctly.
#         |   |   |Add: Display program name & version info at start
# 04/27/01|1.3|rag|Add: Debug tracing
# 04/30/01|1.4|rag|Fix: "-e $ftcmd" test failed if quotes exist in
#         |   |   |     the filename.
# 05/01/01|1.5|rag|Fix: The cleanup of existing $outdir fails.
#         |   |   |Fix: Copying of *Error files fails
# 05/12/01|1.6|rag|Add: Copy the *.properties files
# 09/14/01|1.7|rag|Add: Use ActiveState PerlApp to build "exe".
#         |   |   |     Have program check for *.exe as pgm extension
#         |   |rag|Add: Copy the WASPM* files from $TEMP
#         |   |rag|Add: Use File::Basename to parse $0  
#         |   |rag|Add: Use Cwd to determine present working directory
#---------------------------------------------------------------------
  use Cwd;                             #
  $Version   = "1.7";                  # Version number
  $Tracing   = 0x0000;                 # Trace settings = none
  $DEBUGENTRY= 0x0001;                 # Function entry
  $DEBUGEXIT = 0x0002;                 # Function exit
  $DEBUGFILES= 0x0004;                 # Function - internal filenames
  $DEBUGCMDS = 0x0008;                 # Function - internal commands
#---------------------------------------------------------------------
# Usage() - Display program usage information.
#---------------------------------------------------------------------
  sub Usage {                         
    #-----------------------------------------------------------------
    # What is the "base" program name?   More importantly, what is the
    # "actual" program name?  The result of this test should be the
    # actual program name for the Usage information.
    #-----------------------------------------------------------------
    # Note: It was determined that spending time parsing $0 to try and
    #       guess exacltly which form of this script was invoked would
    #       very difficult, and of insufficient value to make the
    #       effort worthwhile.
    #-----------------------------------------------------------------
      print "Usage: perl WPM_PDtool.pl logdir\n\n";
      print "Where: logdir = Path to Payment Manager log directory\n\n";
      print "A Windows example:\n";
      print "  perl WPM_PDtool.pl \"C:\\Program Files\\IBM\\PaymentManger\\log\"\n";
  }

#---------------------------------------------------------------------
# nyi() - Display "Not Yet Implemented" message
#---------------------------------------------------------------------
  sub nyi {                            #
      print "Not Yet Implemented\n";   #
  }                                    #

#---------------------------------------------------------------------
# PgmName() - Parse $0, and return only the program name portion
#
# Assumptions: "/" and "\\" are not valid parts of the program name.
#---------------------------------------------------------------------
  sub PgmName {                        #
      use File::Basename;              #
      print "PgmName() \$0 = $0\n"     if ( $Tracing & $DEBUGENTRY );
      my @suffixes = ( ".exe", ".pl" );
      my ( $pgm, $path, $ext ) = fileparse( $0, @suffixes );
      print "PgmName() result: $pgm\n" if ( $Tracing & $DEBUGEXIT );
      return $pgm;                     #
  }                                    #

#---------------------------------------------------------------------
# PgmDir() - Parse $0, and return that portion identifying the
#            directory containing the program.
# Note: Result will include trailing "\\"
# Assumptions: "/" and "\\" are not valid parts of the program name.
#---------------------------------------------------------------------
  sub PgmDir {                         #
      my $dir = $0;                    # Start with $0
      print "PgmDir() \$0 = $0\n" if ( $Tracing & $DEBUGENTRY );
      my $last = rindex( $dir, "\\" ); # Locate final backslash
      if ( -1 != $last ) {             # Does one exist?
          $dir = substr( $dir, 0, $last + 1 );
      } else {                         #
          $dir = cwd;                  #
          $dir .= "\\" if ( substr( $dir, -1 ) ne "\\" );
      }                                #
      print "PgmDir() result: $dir\n" if ( $Tracing & $DEBUGEXIT );
      return $dir;                     #
  }                                    #

#---------------------------------------------------------------------
# ExpandName( $name )
#   Role : Fully expand a Windows short name to its equivalent long form
# Example: C:\PROGRA~1\IBM\PAYMEN~1 -> C:\Program Files\IBM\PaymentManager
#  Parms : $name
#  Result: Expanded Windows directory name
#  Usage : $name = ExpandName( $name );
#---------------------------------------------------------------------
  sub ExpandName {                     #
      my ( $name ) = @_;               # Start with user parameter
      $name =~ s/\"//g;                # Remove " characters
      print "ExpandName( \"$name\" )\n" if ( $Tracing & $DEBUGENTRY );
      my @dirs = split( /\\/, $name ); # Split on directory seperator
      my $result = "";                 #
    #----------------------------------
    # Expand each sub-directory level name if is using short name form
    #----------------------------------
      for my $dir ( @dirs ) {          #
          open ( DIR, "dir /a:d /x \"$result\"|" );
          while ( my $here = <DIR> ) {
              if ( $here =~ /<DIR>/ ) {
                  my @fields = split( /  */, $here, 5 );
                  chomp( $fields[ 4 ] );
                  $dir = $fields[ 4 ] if ( $fields[ 3 ] eq $dir );
              }
          }                            #
          close( DIR );                #
          $result .= $dir . "\\";    #
      }                                #
      chop( $result );                 # Remove trailing backslash
      print "ExpandName() result: $result\n" if ( $Tracing & $DEBUGEXIT );
      return $result;                  #
  }                                    #

#---------------------------------------------------------------------
# PMrootdir( $filename )
# Parms : $filename = Registry information to be parsed
# Result: Payment Manager root (install) directory
# Usage : $rootdir = PMrootdir( "ibmsw.out" );
#   Note: Guarantee that result has trailing backslash
# Update: 04/26/01 - If WPM wasn't installed, this would infinite loop
#---------------------------------------------------------------------
  sub PMrootdir {                      #
      my ( $filename ) = @_;           # Start with user parameter
      print "PMrootdir( \"$filename\" )\n" if ( $Tracing & $DEBUGENTRY );
      my $result = "";                 #
      my $line   = "";                 # current line from reg file
    #----------------------------------
    # Parse the registry information.
    # Note: We are looking for the section with a name like one of:
    #   [HKEY_LOCAL_MACHINE\SOFTWARE\IBM\Payment Server\1.2]
    #   [HKEY_LOCAL_MACHINE\SOFTWARE\IBM\Payment Manager\2.1]
    #----------------------------------
      open ( IBMSW, "<$filename" );    # Locate the Payment Svr|Mgr
      while ( chomp( $line = <IBMSW> ) ) { 
          last if ( $line =~ /\[.*Payment (Server|Manager)\\.*]/ );
      }                                #
    #----------------------------------
    # Now, we've either found the Payment Server/Manager section, or,
    # it isn't installed.  If we found the section, we need to locate
    # the installation directory (IDIR).
    #----------------------------------
      if ( $line ne "" ) {             #
          while ( chomp( $line = <IBMSW> ) ) { 
              last if ( ( $result ne "" ) || ( $line eq "" ) );
              $result = $line if ( $line =~ /IDir/ );
          }
      }
      close( IBMSW );
    #----------------------------------
    # Result will either be empty, or of the form:
    # "IDir"="c:\\Program Files\\IBM\\PaymentManager\\"
    #----------------------------------
      if ( $result ne "" ) {           #
          $result =~ s/\"//g;          # Remove quotes (")
          $result =~ s/.*=//;          # Remove keyword & "="
          $result =~ s/\\\\/\\/g;      # rag: 04/17 - delete doubles
          $result .= "\\" if ( substr( $result, -1 ) ne "\\" );
      }                                #
      print "PMrootdir() result: $result\n" if ( $Tracing & $DEBUGEXIT );
      return $result;                  #
  }                                    #

#---------------------------------------------------------------------
# LineCount( $filename )
# Parms : $filename = Name of file for which linecount is determined.
# Result: The number of newline characters found in specified file.
# Usage : $lines = LineCount( $InputFile );
#---------------------------------------------------------------------
  sub LineCount {                      #
      my ( $filename ) = @_;           # User specified filename
      print "LineCount( \"$filename\" );\n" if ( $Tracing & $DEBUGENTRY );
      my $lines  = 0;                  # Number of "\n" chars found
      my $buffer = 0;                  # buffer for data read
      open( FILE, $filename ) or die "Can't open $filename: $!";
      while ( sysread FILE, $buffer, 4096 ) {
          $lines += ($buffer =~ tr/\n//);
      }                                #
      close FILE;                      #
      print "LineCount() result: $lines;\n" if ( $Tracing & $DEBUGEXIT );
      return $lines;                   #
  }                                    #

#---------------------------------------------------------------------
# Tail( FILEHANDLE, $linecount, $filename )
# Parms : FILEHANDLE = filehandle of output file
#         $linecount = number of lines to be written (to outputfile)
#         $filename  = name of input file
# Result: The specified number lines from the end of the input file
#         are written (appended) to the given filehandle
# Usage : Tail( *FILEHANDLE, 10, $sourcefile );
#---------------------------------------------------------------------
  sub Tail {                           #
      my ( $fh, $linecount, $filename ) = @_;
      $filename = "\"$filename\"" if ( $filename =~ / / );
      print "Tail( $linecount, $filename );\n" if ( $Tracing & $DEBUGENTRY );
      my $lines  = LineCount( "$filename" ) - $linecount;
      $lines = 1 if ( $lines < 1 );
      my $data = 0;                    # Current data record
      open( FILE, $filename ) or die "Can't open $filename: $!";
      for ( my $i = 0; $i < $lines; $i++ ) {
          $data = <FILE>;              #
      }                                #
      while ( $data = <FILE> ) {       # Copy remainder of file
          print $fh $data;             #
      }                                #
      close FILE;                      #
      print "Tail() - exit\n" if ( $Tracing & $DEBUGEXIT );
  }                                    #

#---------------------------------------------------------------------
# eTillfmt( $PMrootdir, $logdir, $outdir )
# Parms : $PMrootdir = WPM installation/root directory
#         $logdir    = WPM log directory
#         $outdir    = Output directory
# Result: fmttrace of eTillSETMsg* files from $logdir to $outdir
# Usage : eTillfmt( $logdir, $outdir );
#---------------------------------------------------------------------
  sub eTillfmt {                       #
      use DirHandle;                   # Directory manipulation
      my ( $PMrootdir, $logdir, $outdir ) = @_;
      $outdir    =~ s/\"//g;           # Remove any quotes (")
      print "eTillfmt( \"$PMrootdir\", \"$logdir\", \"$outdir\" );\n" if ( $Tracing & $DEBUGENTRY );
      my $tailfile = "$outdir\\eTilltails.out";
      $tailfile = "\"$tailfile\"" if ( $tailfile =~ / / );
      print "\$tailfile = $tailfile\n" if ( $Tracing & $DEBUGFILES );
      open( TAILFILE, ">$tailfile" ) or die "Can't open $tailfile: $!";
      chdir( $logdir );                # Make logdir current directory
      my $dh = new DirHandle ".";      # Process files therein
      my $files, @files, %files;       #
      if ( defined $dh ) {             # Is there anything to process?
          @files = ();                 # list of files to be processed
          while ( defined( $_ = $dh -> read ) ) {
             my $name = $_;            #   current filename
             push @files, $name if ( -f $name && $name =~ /eTillSETMsg*/ );
          }                            #
        #------------------------------
        #   At this point, @files contains the names of the files to
        # be processed.  What we want to do, however, is to process
        # them in chronological order (i.e., oldest first).  This will
        # allow the people who have to look at the processed files
        # to quickly and easily determine which file contains the data
        # of most interest to them, based upon what occurred, when.
        #   To do this, we create a hash, indexed by the file
        # modification time.
        #------------------------------
          %files = {};                 # Process chronologically
          foreach $file ( @files ) {   #
              $files{ $file } = ( stat( $file ) )[9];
          }                            #
          chdir( $PMrootdir );         # cd to PMrootdir for fmttrace
        #------------------------------
        #   Now that we have this hash created, we use the indexes to 
        # determine the order in which the files should be processed
        # (i.e., oldest first).  To make the processing results easy
        #  for everyone to understand, we will make the modification
        #  time part of the generated (output) filename.
        #------------------------------
          my @inorder = sort { $files{ $a } <=> $files{ $b } } keys %files;
          for ( my $i = 1; $i <= $#inorder; $i++ ) {
              my $mtime = $files{ $inorder[ $i ] };
              my ( $sec, $min, $hour, $mday, $mon,
                  $year, $wday, $yday, $isdst) = localtime( $mtime );
            #--------------------------
            # Here is where we convert the WPM generated name, e.g.,
            # eTillSETMsgDump.21 to a name containing the date/time
            # stamp Dump.21.2001-02-29.12-34-56 if the file was last
            # modified at 12:34:56 on 02/29/2001 (mon/day/year).
            # Note that Windows does not allow filenames to contain
            # any of the following characters: \ / : * ? " < > |
            #--------------------------
              my $name = $inorder[ $i ];
              $name =~ s/eTillSETMsg//;# eTillSETMsgDump.## -> Dump.##
              $name .= sprintf( ".%04d-%02d-%02d.%02d-%02d-%02d",
                  $year+1900, $mon+1, $mday, $hour, $min, $sec );
              print "\$name = $name\n" if ( $Tracing & $DEBUGFILES );
              my $cmd = "lib\\fmttrace \"$logdir\\" . $inorder[ $i ] . "\" >\"$outdir\\$name\"";
              print "fmttrace cmd: $cmd " . $inorder[ $i ] . " >$name\n" if ( $Tracing & DEBUGCMDS );
              system( "$cmd" );
            #--------------------------
            # Now that we've formatted the current Dump/Trace file, we
            # want to take the last 20 lines of it, and write it to
            # the eTilltails.out file.  First, however, we want to put
            # a delimiter line in place which contains the original
            # filename, and the date and timestamp of that file.
            #--------------------------
              printf TAILFILE  "------ %s modified on %04d/%02d/%02d at %02d:%02d:%02d ------\n",
                  $inorder[ $i ], $year+1900, $mon+1, $mday, $hour, $min, $sec;
              Tail( *TAILFILE, 20, "$outdir\\$name" );
          }                            #
          undef $dh;                   #
      }                                #
      close TAILFILE;                  #
      print "eTillfmt() - exit\n" if ( $Tracing & $DEBUGEXIT );
  }                                    #

#---------------------------------------------------------------------
# rel2ab( $Parentdir, $dir )
#   Role: If specified $dir is relative, convert it to absolute by
#         prepending the specified prefix directory.
#  Parms: $Parentdir = Parent directory to be used for conversion
#         $dir       = Directory to be checked
# Result: An absolute, fully-qualified directory
# Usage : $dir = rel2ab( $PMrootdir, $dir );
#  Notes: If $Parentdir doesn't end with "\\", it will be appended.
#         If quotes are present, they will be removed.
#         If the result requires quotes, they will be added
#---------------------------------------------------------------------
  sub rel2ab {
      my ( $Pdir, $dir ) = @_;
      $Pdir =~ s/\"//g;                # Remove " character(s)
      $Pdir .= "\\" if ( substr( $Pdir, -1 ) ne "\\" );
      $dir =~ s/\"//g;                 # Remove " character(s)
      print "rel2ab( $Pdir, $dir )\n" if ( $Tracing & $DEBUGENTRY );
      if ( $dir !~ /^(([a-zA-Z][:])?\\)/ ) {
          $dir = $Pdir .  $dir;
      }
      $dir = "\"$dir\"" if ( $dir =~ / / );
      print "rel2ab() - result: $dir\n" if ( $Tracing & $DEBUGEXIT );
      return $dir;
  }

#---------------------------------------------------------------------
# Windows( $logdir )
# Parms : $logdir = Payment Manager trace & log directory
#---------------------------------------------------------------------
  sub Windows {                        # What flavor of Windows?
      my ( $logdir ) = @_;             # What logdir was specified?
      print "Windows( \"$logdir\" )\n" if ( $Tracing & $DEBUGENTRY );
      my $cmd, $msg, $idir, $outfile, $dir;
      my $delim = "#" . "-" x 75 . "\n"; # Message delimiter line
    #----------------------------------
    # First, we load the routines used to dump the Registry, and
    # Format the Traces
    # 04/26/01 rag - Note, handle the situation where WPM_PDtool.pl is
    #                not in the current directory.
    #                It is presumed that both this script, and
    #                WinRegDump.pl are in the same directory.
    #----------------------------------
      my $pgmdir = PgmDir();             # Dir containing WPM_PDtool.pl
      print "require $pgmdir . \"WinRegDump.pl\";\n" if ( $Tracing & $DEBUGCMDS );
      require $pgmdir . "WinRegDump.pl"; # Load WinRegDump routine(s)
    #----------------------------------
    # Is the user specified logdir is absolute, or relative?  If it's
    # relative, prepend the current working directory.
    #----------------------------------
      $dir = cwd;
      $logdir = rel2ab( $dir, $logdir );
      $logdir = ExpandName( $logdir ); # Convert short 8.3 to long form
    #----------------------------------
    # Next, we find out which version of Windows is being used.
    #----------------------------------
      my $ver = `cmd /c ver`;          #
      if ( $ver =~ /Windows 2000/ ) {  # Windows 2K specific command
          $cmd  = "c:\\WINNT\\system32\\dllcache\\msinfo32 /report sysinfo.txt";
          $msg  = $delim;
          $msg .= "# Warning: The following command may take a few minutes to complete.\n";
          $msg .= $delim;
          $msg .= "msinfo32 >sysinfo.txt\n";
      } elsif ( $ver =~ /Windows NT/ ) { # Windows/NT specific command
          $cmd = "winmsd /a /f";       # Send complete report to file
          $msg = "winmsd >sysinfo.txt\n";
      } else {                         #
          chomp( $ver = reverse $ver );# Strip leading  \n char(s)
          chomp( $ver = reverse $ver );# Strip trailing \n char(s)
          print "Unrecognized windows version: \"$ver\"\n";
          return;                      # 
      }                                #
    #----------------------------------
    # Next, we find out where $TEMP is, and our own program name.
    #----------------------------------
      use Env qw( TEMP );              # Determine the value of $TEMP
      my $pgm = PgmName();             # What is our program name?
      my $outdir = ExpandName( "$TEMP\\$pgm" );
      print "\$outdir: $outdir\n" if ( $Tracing & $DEBUGFILES );
    #----------------------------------
    # Using this info, we create a directory for all of the info being
    # gathered by this program.
    # rag: 04/15/01 - Check for existance of $outdir file/directory
    #----------------------------------
      if ( -e $outdir ) {              # Does it already exist?
          if ( -d $outdir ) {          #   Yes, is it a directory?
              print "del /q \"$outdir\"\n" if ( $Tracing & $DEBUGCMDS );
              open( DEL, "del /q \"$outdir\" 2>&1 |" );
              close( DEL );            #   remove the contents.
          } else {                     #
              print "unlink $outdir\n" if ( $Tracing & $DEBUGCMDS );
              unlink $outdir;          #   It's a file, delete it.
              print "mkdir  $outdir\n" if ( $Tracing & $DEBUGCMDS );
              mkdir "$outdir";         #   Create a directory ...
          }                            #
      } else {                         #
          print "mkdir  $outdir\n" if ( $Tracing & $DEBUGCMDS );
          mkdir "$outdir";             # Create a directory for data
      }                                #
      print $delim;                    #
      print "# Creating the following directory, to hold the data.\n";
      print "# $outdir\n";             #
      print $delim;                    #
      chdir "$TEMP";                   # $TEMP should have *install.trace* files 
    #----------------------------------
    # The first thing we copy in are the *install.trace* files.  We
    # look for how many of these files were copied by searching the
    # result for a line that looks like: "   0 files(s) copied."
    #----------------------------------
      print "copying install.trace files\n";
      print "copy *install.trace* $pgm\n" if ( $Tracing & $DEBUGCMDS );
      open( COPY, "copy *install.trace* $pgm 2>&1 |" ) or die "Can't fork copy command: $!";
      my $copy = "0";                  # Default filecount
      while ( <COPY> ) {               # Process output of copy command
        chomp;                         # Discard trailing newline
        $copy = $_ if ( /copied.$/ );  # Keep line with filecount
      }                                #
      close( COPY );                   # Ignore any close() failures
    #----------------------------------
    # Parse this result, and if no files were copied, display a message.
    #----------------------------------
      $copy =~ s/^  *([0-9]+) file.*$/$1/;# Keep only the filecount
      if ( $copy == 0 ) {              # Were any files copied?
          print $delim;
          print "# Warning: No install.trace files were found.  They may have been deleted.\n";
          print $delim;
      }                                #
    #----------------------------------
    # Next, try to copy in any WASPM* files...
    #----------------------------------
      print "copying any WASPM* files that are present\n";
      print "copy WASPM* $pgm\n" if ( $Tracing & $DEBUGCMDS );
      open( COPY, "copy WASPM* $pgm 2>&1 |" ) or die "Can't fork copy command: $!";
      my $copy = "0";                  # Default filecount
      while ( <COPY> ) {               # Process output of copy command
        chomp;                         # Discard trailing newline
        $copy = $_ if ( /copied.$/ );  # Keep line with filecount
      }                                #
      close( COPY );                   # Ignore any close() failures
    #----------------------------------
    # Make $TEMP\$pgm the current directory.
    #----------------------------------
      chdir "$pgm";                    #
      print "$msg";                    # Note: $msg contains ending \n
    #----------------------------------
    # On Win/NT, we have to use the winmsd command, which does not
    # allow us to specify the name of the output file.  So, we first
    # create a temp directory which will be used to hold the file
    # created by winmsd.  The reason for deleting the sysinfo.txt, is
    # so the subsequent move command won't fail.
    #----------------------------------
      if ( $cmd =~ /winmsd/ ) {        # Handle winmsd differently
          unlink "sysinfo.txt";        # Delete existing sysinfo.txt
          mkdir  "winmsd.out";         # Temp sub-directory for result
          chdir  "winmsd.out";         # Make sub-direcroty current
      }                                #
      system( $cmd );                  # Execute msinfo32/winmsd cmd
    #----------------------------------
    # On Win/NT, we now move/rename this file to the parent directory,
    # using the preferred name (sysinfo.txt).  In addition, we delete
    # the temporary sub-directory, which is no longer required.
    #----------------------------------
      if ( $cmd =~ /winmsd/ ) {        # 
          print "move * ..\\sysinfo.txt\n" if ( $Tracing & $DEBUGCMDS );
          system( "move * ..\\sysinfo.txt >nul:" );
          chdir "..";                  #
          rmdir "winmsd.out";          # Remove temp sub-directory
      }                                #
    #----------------------------------
    # We also want to save the network configuration information.
    #----------------------------------
      $cmd = "ipconfig /all >ipconfig.out";
      print "$cmd\n";                  #
      system( $cmd );                  #
    #----------------------------------
    # and, the network status information
    #----------------------------------
      $cmd = "netstat -a  >netstat_a.out";
      print "$cmd\n";                  #
      system( $cmd );                  #
      $cmd = "netstat -an >netstat_an.out";
      print "$cmd\n";                  #
      system( $cmd );                  #
    #----------------------------------
    # Next, we output the \HKLM\Software\IBM subtree from the registry
    #----------------------------------
      WinRegDump();                    #
    #----------------------------------
    # Now, we want to do some processing and collecting of the log and
    # trace files found in the PaymentManager log directory.  To do
    # this, things are greatly simplified if the current working dir
    # is made to be the PaymentManager installation (root) directory.
    #----------------------------------
      $! = 1;
      chop( $logdir ) if ( substr( $logdir, -1 ) eq "\\" );
      print "logdir: $logdir\n" if ( $Tracing & $DEBUGFILES );
      $idir = PMrootdir( "$TEMP\\$pgm\\ibmsw.out" );
      print "PMrootdir: $idir\n" if ( $Tracing & $DEBUGFILES );
      if ( $idir eq "" ) {
        print "WebSphere Payment Manager doesn't appear to be installed.\n";
      } else {
        #-------------------------------------------------------------
        # Now that we have the fully qualified path to the Payment
        # Manager log directory, we can copy the eTillError & PSError
        # files over to the sub-directory we created under $TEMP.
        #-------------------------------------------------------------
          print "copying eTillError & PSError files\n";
          my $ifile = "$logdir\\*Error";
          $ifile = "\"$ifile\"" if ( $ifile =~ / / );
          my $ofile = "$TEMP\\$pgm";
          $ofile = "\"$ofile\"" if ( $ofile =~ / / );
          $cmd = "copy $ifile $ofile";
          print "$cmd\n" if ( $Tracing & $DEBUGCMDS );
        #-------------------------------------------------------------
        # We don't really care to know how many files got copied...
        #-------------------------------------------------------------
          open( COPY, "$cmd 2>&1 |" ); #
          while ( <COPY> ) {           # Process output of copy command
            print if ( $Tracing & $DEBUGCMDS );
          }
          close( COPY );               # Ignore any close() failures
        #-------------------------------------------------------------
        # rag: 05/12/01 - These files contain lots of useful info.
        # Next, we want to copy the *.properties files.
        #-------------------------------------------------------------
          print "copying *.properties files\n";
          $cmd = "copy $idir" . "*properties $ofile";
          print "$cmd\n" if ( $Tracing & $DEBUGCMDS );
          open( COPY, "$cmd 2>&1 |" ) or die "Can't fork copy command: $!";
          my $copy = "0";              # Default filecount
          while ( <COPY> ) {           # Process output of copy command
            chomp;                     # Discard trailing newline
            $copy = $_ if ( /copied.$/ );  # Keep line with filecount
          }                            #
          close( COPY );               # Ignore any close() failures
        #-------------------------------------------------------------
        # rag: 04/15/01 - Build a WPM_Format.cmd file to do the
        #                 FormatTrace and FormatServletTrace commands
        #-------------------------------------------------------------
        # Now, we want to format the trace and servlet trace files...
        # To do this, I want to use the same command used in the
        # FormatTrace.cmd and FormatServletTrace.cmd files which were
        # installed by WPM.  Search for the invocation of the JVM used
        # to execute the Format class file.
        #-------------------------------------------------------------
          my $ftcmd = $idir . "FormatTrace.cmd";
          print "ftcmd: $ftcmd\n" if ( $Tracing & $DEBUGCMDS );
          if ( -e $ftcmd ) {           # Does FormatTrace.cmd exist?
              $cmd = "copy $ftcmd \"$TEMP\\$pgm\"";
              print "$cmd\n" if ( $Tracing & $DEBUGFILES );
              open( COPY, "$cmd 2>&1 |" ); #
              close( COPY );               # Ignore any close() failures
              open( FTCMD, $ftcmd ) or die "Can't open $ftcmd file: $!";
              $cmd = "";               # JVM invocation cmd
              while ( <FTCMD> ) {      # Process FormatTrace.cmd file
                  chomp;               # Discard trailing newline
                  $cmd = $_ if ( /-classpath/i );
              }                        #
              close( FTCMD );          # Ignore any close() failures
            #---------------------------------------------------------
            # At this point, $cmd will be of the form:
            #    0         1          2         3       4     5   6   7
            # jvmpath -classpath $CLASSPATH javaclass parm1 parm2 > parm3
            #---------------------------------------------------------
              $cmd =~ s/^  *//;        # Remove any leading ...
              $cmd =~ s/  *$//;        #   ... trailing ...
              print "FormatTrace cmd:\n$cmd\n\n" if ( $Tracing & $DEBUGCMDS );
            #---------------------------------------------------------
            # What we want now, is to split this command into its
            # component pieces.  For example:
            #   $cmd[ 0 ] = ".\jre\bin\jre"
            #   $cmd[ 1 ] = "-classpath"
            #   $cmd[ 2 ] = ".\jre\lib\rt.jar;.\jre\lib\i18n.jar;.\eTillClasses.zip"
            #   $cmd[ 3 ] = "com.ibm.etill.framework.log.FormatTrace"
            #   $cmd[ 4 ] = "%1\eTillTrace1.log"
            #   $cmd[ 5 ] = "%1\eTillTrace2.log"
            #   $cmd[ 6 ] = ">"
            #   $cmd[ 7 ] = "%2"
            # What I want to do at this point, is to convert relative
            # paths to absolute paths, and to use double-quotes around
            # any paths which contain imbedded blanks.  It would be
            # really nice, if we were able to do a simple:
            #   @cmd = split( /\s/, $cmd );
            # Unfortunately, If any of the pieces (e.g., jvmpath,
            # classpath, etc.) have any imbeded blanks, this won't
            # work.  So, I have to do my own parsing of this command
            # string into the cmd array, as shown above.
            #---------------------------------------------------------
              my @cmd   = ();                    # parsed cmd array
              my $piece = "";                    # Current string piece
              my $quote =  0;                    # Processing a quoted string?
              my $skip  =  1;                    # Skip whitespace?
              foreach ( split( //, $cmd ) ) {    # Foreach char in $cmd string
                  next if ( /\s/ && $skip );     # Skipping whitespace?
                  if ( /\s/ ) {                  # Whitespace?
                      if ( $quote ) {            #   Imbedded quote?
                          $piece .= $_;          #     Yes, save this char 
                          $skip   = 0;           #          & turn skip off
                      } else {                   #     No,  separator
                          print "push( \@cmd, \"$piece\" );\n" if ( $Tracing & $DEBUGCMDS );
                          push( @cmd, $piece );  #   No, end of piece indicated
                          $piece = "";           #   Init for next string
                          $skip  = 1;            #   & turn skip on
                      }                          #
                  } elsif ( /\"/ ) {             # Quote (") char?
                      $quote  = !$quote;         #   Yes, toggle the quote flag
                      $piece .= $_;              #   & save this char 
                      $skip   = 0;               #   & turn skip off
                  } else {                       # Not whitespace or quote
                      $piece .= $_;              #   Save the char 
                      $skip   = 0;               #   & turn skip off
                  }                              #
              }                                  #
              if ( $piece ne "" ) {              #
                  print "push( \@cmd, \"$piece\" );\n" if ( $Tracing & $DEBUGCMDS );
                  push( @cmd, $piece );          #
              }                                  #
              my $jvm = rel2ab( $idir, $cmd[ 0 ] );
              print "\$jvm = $jvm\n" if ( $Tracing & $DEBUGFILES );
            #---------------------------------------------------------
            # Now, do the same stuff to the CLASSPATH directories
            # (i.e., convert relative paths to absolute, and add quotes
            # to those directories containing imbedded blanks).
            #---------------------------------------------------------
              my $cp  = $cmd[ 2 ];     # CLASSPATH value
              $cp =~ s/\"//g;          #   ... without quote (") chars
              my @cp  = split( /;/, $cp );
              foreach $dir ( @cp ) {   # process each CLASSPATH dir
                  $dir = rel2ab( $idir, $dir );
              }                        #
              $cp = join( ';', @cp );  # reconstituted CLASSPATH
            #---------------------------------------------------------
            # The javaclass to be executed is very straightforward.
            #---------------------------------------------------------
              my $class = $cmd[ 3 ];   # Javaclass to be executed
            #---------------------------------------------------------
            # Now, use all of this information to create a new .cmd
            # file which will really execute both FormatTrace, and
            # FormatServletTrace commands.
            #---------------------------------------------------------
              my $format = "$TEMP\\$pgm\\WPM_Format.cmd";
              open( FORMAT, ">$format" ) or die "Can't open $format file: $!";
              print FORMAT "\@ECHO OFF\n";
              print FORMAT "rem **********\n";
              print FORMAT "rem Filename: $format\n";
              print FORMAT "rem  Created: ". ( scalar localtime ) . " by $pgm\n";
              print FORMAT "rem  Purpose: Execute WPM FormatTrace and";
              print FORMAT "FormatServletTrace commands\n";
              print FORMAT "rem **********\n";
              $logdir  =~ s/\\\\/\\/g; # Change "\\" -> "\"
              my $ifile1 = "$logdir\\eTillTrace1.log";
              $ifile1  = "\"$ifile1\"" if ( $ifile1 =~ / / );
              my $ifile2 = "$logdir\\eTillTrace2.log";
              $ifile2  = "\"$ifile2\"" if ( $ifile2 =~ / / );
              $outfile = "$TEMP\\$pgm\\FormatTrace.out";
              $outfile = "\"$outfile\"" if ( $outfile =~ / / );
              $cmd = "$jvm -classpath $cp $class $ifile1 $ifile2 > $outfile";
              print FORMAT "$cmd\n";
              my $ifile1 = "$logdir\\PSTrace1.log";
              $ifile1  = "\"$ifile1\"" if ( $ifile1 =~ / / );
              my $ifile2 = "$logdir\\PSTrace2.log";
              $ifile2  = "\"$ifile2\"" if ( $ifile2 =~ / / );
              $outfile = "$TEMP\\$pgm\\FormatServletTrace.out";
              $outfile = "\"$outfile\"" if ( $outfile =~ / / );
              $cmd = "$jvm -classpath $cp $class $ifile1 $ifile2 > $outfile";
              print FORMAT "$cmd\n";
              close( FORMAT );         # Ignore any close() failures
              $format = "\"$format\"" if ( $format =~ / / );
              system( $format );
          } else {                     # FormatTrace.cmd doesn't exist
              print $delim;
              print "# File not found: $ftcmd\n\n";
              print "# Can't execute FormatTrace & FormatServletTrace commands\n";
              print $delim;
          }
        #-------------------------------------------------------------
        # ... and the eTillSETMsg* files
        #-------------------------------------------------------------
          eTillfmt( $idir, $logdir, "$TEMP\\$pgm" );
      }                                #
      print "Windows() - exit\n" if ( $Tracing & $DEBUGEXIT );
  }                                    #

#---------------------------------------------------------------------
# AIX()
#---------------------------------------------------------------------
  sub AIX {                            #
      print "AIX\n";                   #
      nyi();                           #
  }                                    #

#---------------------------------------------------------------------
# Solaris()
#---------------------------------------------------------------------
  sub Solaris {                        #
      print "Solaris\n";               #
      nyi();                           #
  }                                    #

#---------------------------------------------------------------------
# main()
#---------------------------------------------------------------------
  $| = 1;                              # Auto-flush STDOUT
  $argc = $#ARGV + 1;                  # How many parms were specified?
  print PgmName() . " $Version\n";     # Display program name & version
  if ( 1 != $argc ) {                  # It had better be 1
      Usage();                         #
  } else {                             #
      $logdir = $ARGV[ 0 ];            #
      if ( ! -d $logdir ) {            #
          print "$logdir is not a directory.\n";
          Usage();                     #
      } else {                         #
          if ( $^O eq "MSWin32" ) {    # 1st level OS determination
              Windows( $logdir );      # Windows specific processing
          } elsif ( $^O eq "aix" ) {   #
              AIX( $logdir );          # AIX     specific processing
          } elsif ( $^O eq "solaris" ) {
              Solaris( $logdir );      # Solaris specific processing
          } else {                     #
              print "Unrecognized operating system\n";
          }                            #
      }                                #
  }                                    #
